﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml.Serialization;

namespace Orxe2Setup
{
    public class FileHandlingOfWebConfig
    {
        public string Path { get; set; }
        public string LucenePath { get; set; }
        public string EnvironmentToPoint { get; set; }
        public string BranchNumber { get; set; }

        public FileHandlingOfWebConfig(string basePathOfFile, string branchNumber, string environmentToPoint)
        {
            // C:\Vicky\Connexions\AppDev\Code\ORXe\201909\ORXe2.0\02.UI\Portal\Web.Config
            this.Path = basePathOfFile + @"\AppDev\Code\ORXe\" + branchNumber + @"\ORXe2.0\02. UI\Portal\Web.Config";
            this.EnvironmentToPoint = environmentToPoint;
            this.BranchNumber = branchNumber;
            // C:\Vicky\Connexions\AppDev\Code\ORXe\201909\ORXe2.0\02. UI\Portal\AutoSuggestIndex
            this.LucenePath = basePathOfFile + @"\AppDev\Code\ORXe\" + branchNumber + @"\ORXe2.0\02. UI\Portal\AutoSuggestIndex";
        }

        public void HandleFile()
        {
            Console.WriteLine("File Present - " + File.Exists(this.Path));
            
            if (File.Exists(this.Path))
            {
                FileInfo fileInfo = new FileInfo(this.Path);
                fileInfo.IsReadOnly = false;
                try
                {
                    Console.WriteLine("Making required changes in " + this.BranchNumber + "'s Web.Config file. Please wait...");
                    string[] lines = File.ReadAllLines(this.Path);
                    CopyTheFileAndBlankIt(this.Path);
                    ChangeSettings(lines);
                    Console.WriteLine("Rewriting Done!!!");

                    File.Delete(this.Path + "_TEMP");

                   
                }
                catch (Exception e)
                {
                    File.Delete(this.Path);
                    File.Copy(this.Path +"_TEMP", this.Path);
                    File.Delete(this.Path + "_TEMP");
                    throw e;
                }
            }
            else
            {
                Console.WriteLine("Error: File not found. please enter correct path.!!!");
                Console.ReadLine();
            }
        }
        
        public void ChangeSettings(string[] lines)
        {
            try
            {
                bool isAppSettingOpen = false;
                bool isCmTemplarOpen = false;
                bool isClientOpen = false;
                bool isConnectionStringOpen = false;

                foreach (var line in lines)
                {

                    if (isAppSettingOpen || isCmTemplarOpen || isClientOpen) {
                        AlterSettingsLines(line);
                    }
                    else if (isConnectionStringOpen) {
                        AlterConnectionStringLines(line);
                    }
                    else {
                        WriteToFile(line);
                    }

                    if (line.Trim().ToLower().Equals("<cmtemplar>")) {
                        isCmTemplarOpen = true;
                    }

                    if (line.Trim().ToLower().Equals("</cmtemplar>") && isCmTemplarOpen) {
                        isCmTemplarOpen = false;
                    }

                    if (line.Trim().ToLower().Equals("<connectionstrings>"))
                    {
                        isConnectionStringOpen = true;
                    }

                    if (line.Trim().ToLower().Equals("</connectionstrings>") && isConnectionStringOpen)
                    {
                        isConnectionStringOpen = false;
                    }

                    if (line.Trim().ToLower().Equals("<appsettings>")) {
                        isAppSettingOpen = true;
                    }

                    if (line.Trim().ToLower().Equals("</appsettings>") && isAppSettingOpen) {
                        isAppSettingOpen = false;
                    }
                    
                    if (line.Trim().ToLower().Equals("<client>")) {
                        isClientOpen = true;
                    }

                    if (line.Trim().ToLower().Equals("</client>") && isClientOpen) {
                        isClientOpen = false;
                    }

                }
            }
            catch (Exception e)
            {
                throw e;
            }

        }

        public void AlterSettingsLines(string line)
        {
            if (line.Trim().Contains("-core.loyaltytravelservices.com"))
            {
                string[] arrayofChanginglines = line.Split("-core.loyaltytravelservices.com");

                string newLine = arrayofChanginglines[0].Split("https://")[0] + "https://" + this.EnvironmentToPoint + "-core.loyaltytravelservices.com" + arrayofChanginglines[1];

                WriteToFile(newLine);

            } else if (line.Trim().Contains("<add key=\"IndexPath\"")|| line.Trim().Contains("<add key=\"AirlineIndexPath\"")) {
               
                // <add key="IndexPath" value="C:\_LOGS\AutoSuggestIndex"/>
                // <add key="AirlineIndexPath" value="C:\_LOGS\AutoSuggestIndex\Airline"/>

                string[] arrayofChanginglines = line.Split("value=\"");

                string newLine = arrayofChanginglines[0] + "value=\"" + this.LucenePath + arrayofChanginglines[1].Split("AutoSuggestIndex")[1];
                WriteToFile(newLine);

            } else {
                WriteToFile(line);
            }
        }

        public void AlterConnectionStringLines(string line)
        {

            // <add name="TemplarDB" connectionString="Data Source=.;initial catalog=txTEMPLARRESPONSIVE;User Id=sa;password=test123!@#" providerName="System.Data.SqlClient" />

            //<add name="FileManagerDB" connectionString="Data Source=.;initial catalog=txTEMPLARRESPONSIVE;User Id=sa;password=test123!@#" providerName="System.Data.SqlClient" />
            //<add name="ExceptionDB" connectionString="Data Source=.;initial catalog=txTEMPLARRESPONSIVE;User Id=sa;password=test123!@#" providerName="System.Data.SqlClient" />
            //<add name="AuditLogDB" connectionString="Data Source=.;initial catalog=txTEMPLARRESPONSIVE;User Id=sa;password=test123!@#" providerName="System.Data.SqlClient" />
            // <add name=\"TemplarDB\" connectionString=\"Data Source=QDAPITemplar.loyaltytravelservices.com\\APISQLTemplar,1433;Integrated Security=True;initial catalog=ctxTEMPLARRESPONSIVE\" providerName=\"System.Data.SqlClient\"/>
            if (line.Contains("Data Source"))
            {
                string[] arrayofChanginglines = line.Trim().Split("Data Source");
                string[] changingLineSecond = arrayofChanginglines[1].Split(";");
                changingLineSecond[0] = ".";
                changingLineSecond[1] = "initial catalog=txTEMPLARRESPONSIVE;User Id=sa;password=test123!@#\" providerName=\"System.Data.SqlClient\"/>";
                changingLineSecond[2] = "";
                string newLine = arrayofChanginglines[0] + "Data Source=" + String.Join(";", changingLineSecond);
                WriteToFile(newLine.TrimEnd(';'));
            }
            else {
                WriteToFile(line);
            }

        }

        public void WriteToFile(string line)
        {
            using (StreamWriter sw = File.AppendText(this.Path))
            {
                sw.WriteLine(line);
            }
        }

        public void CopyTheFileAndBlankIt(string filePath) {

            File.Copy(filePath, filePath + "_TEMP");
            File.WriteAllText(filePath, "");

        }
    }

}
